import ErrorBoundary from "@components/ErrorBoundary";
import definePlugin from "@utils/types";
import { findByPropsLazy, findComponentByCodeLazy, findStoreLazy } from "@webpack";

const Routes = findByPropsLazy('GUILD_MEMBER_VERIFICATION_FOR_HUB', 'QUEST_HOME');

const QuestsStore = findStoreLazy("QuestsStore");

const LinkButton = findComponentByCodeLazy("route:", ",selected:", ",icon:", ",iconClassName:");
const NumberBadge = findComponentByCodeLazy("count:", ",color:", ",disableColor:", ",shape:");

const QuestsButton = () => {
    const activeQuestsNotEnrolled = Array.from(QuestsStore.quests.values()).filter((quest: any) => {
        const expiresAt = new Date(quest.config.expiresAt).getTime();
        const currentTime = Date.now();
        const isExpired = expiresAt <= currentTime;
        const isEnrolled = quest.userStatus?.enrolledAt != null;

        return !isExpired && !isEnrolled;
    });

    return (
        <ErrorBoundary noop>
            <LinkButton
                text="Quests"
                icon={() => <svg aria-hidden="true" role="img" xmlns="http://www.w3.org/2000/svg" width="24" height="24" fill="none" viewBox="0 0 24 24"><path fill="currentColor" d="M7.5 21.7a8.95 8.95 0 0 1 9 0 1 1 0 0 0 1-1.73c-.6-.35-1.24-.64-1.9-.87.54-.3 1.05-.65 1.52-1.07a3.98 3.98 0 0 0 5.49-1.8.77.77 0 0 0-.24-.95 3.98 3.98 0 0 0-2.02-.76A4 4 0 0 0 23 10.47a.76.76 0 0 0-.71-.71 4.06 4.06 0 0 0-1.6.22 3.99 3.99 0 0 0 .54-5.35.77.77 0 0 0-.95-.24c-.75.36-1.37.95-1.77 1.67V6a4 4 0 0 0-4.9-3.9.77.77 0 0 0-.6.72 4 4 0 0 0 3.7 4.17c.89 1.3 1.3 2.95 1.3 4.51 0 3.66-2.75 6.5-6 6.5s-6-2.84-6-6.5c0-1.56.41-3.21 1.3-4.51A4 4 0 0 0 11 2.82a.77.77 0 0 0-.6-.72 4.01 4.01 0 0 0-4.9 3.96A4.02 4.02 0 0 0 3.73 4.4a.77.77 0 0 0-.95.24 3.98 3.98 0 0 0 .55 5.35 4 4 0 0 0-1.6-.22.76.76 0 0 0-.72.71l-.01.28a4 4 0 0 0 2.65 3.77c-.75.06-1.45.33-2.02.76-.3.22-.4.62-.24.95a4 4 0 0 0 5.49 1.8c.47.42.98.78 1.53 1.07-.67.23-1.3.52-1.91.87a1 1 0 1 0 1 1.73Z" className=""></path></svg>}
                route={Routes.QUEST_HOME}
            >
                {activeQuestsNotEnrolled.length > 0 && (
                    <NumberBadge
                        count={activeQuestsNotEnrolled.length}
                    />
                )}
            </LinkButton>
        </ErrorBoundary>
    );
};

export default definePlugin({
    name: "QuestsHomeNav",
    description: "Adds a button to quickly go to quests to the home page",
    authors: [
        {
            id: 644298972420374528n,
            name: "Nick"
        }
    ],
    patches: [
        {
            find: "\"discord-shop\"",
            replacement: {
                match: /"discord-shop"\),/,
                replace: "$&,$self.QuestsButton(),"
            }
        }
    ],
    QuestsButton
});
